"use strict";
/*
 * SPDX-FileCopyrightText: 2021 Zextras <https://www.zextras.com>
 *
 * SPDX-License-Identifier: AGPL-3.0-only
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getXmlSoapFetch = exports.soapFetchV2 = exports.getSoapFetch = void 0;
const lodash_1 = require("lodash");
const fetch_utils_1 = require("./fetch-utils");
const user_agent_1 = require("./user-agent");
const utils_1 = require("./utils");
const constants_1 = require("../constants");
const functions_1 = require("../reporting/functions");
const account_1 = require("../store/account");
const network_1 = require("../store/network");
const utils_2 = require("../store/network/utils");
const fetchNoOp = () => {
    // eslint-disable-next-line @typescript-eslint/no-use-before-define
    (0, exports.getSoapFetch)(constants_1.SHELL_APP_ID)('NoOp', network_1.useNetworkStore.getState().pollingInterval === 500
        ? { _jsns: constants_1.JSNS.mail, limitToOneBlocked: 1, wait: 1 }
        : { _jsns: constants_1.JSNS.mail });
};
const getXmlAccount = (acc, otherAccount) => {
    if (otherAccount) {
        return `<account by="name">${otherAccount}</account>`;
    }
    if (acc) {
        if (acc.name) {
            return `<account by="name">${acc.name}</account>`;
        }
        if (acc.id) {
            return `<account by="id">${acc.id}</account>`;
        }
    }
    return '';
};
const getXmlSession = () => {
    const sessionId = network_1.useNetworkStore.getState().session?.id;
    if (sessionId) {
        return `<session id="${sessionId}"/>`;
    }
    return '';
};
const normalizeContext = ({ notify: rawNotify, ...context }) => {
    const normalizedContext = { ...context, notify: [] };
    if (rawNotify) {
        normalizedContext.notify = (0, lodash_1.map)(rawNotify, (notify) => ({
            ...notify,
            deleted: notify.deleted?.id?.split(',') ?? []
        }));
    }
    return normalizedContext;
};
const handleResponseV2 = (res) => {
    const { noOpTimeout } = network_1.useNetworkStore.getState();
    const { usedQuota } = account_1.useAccountStore.getState();
    clearTimeout(noOpTimeout);
    if (res.Body.Fault) {
        if ((0, lodash_1.find)(['service.AUTH_REQUIRED', 'service.AUTH_EXPIRED'], (code) => code === res.Body.Fault.Detail?.Error?.Code)) {
            if (constants_1.IS_FOCUS_MODE) {
                account_1.useAccountStore.setState({ authenticated: false });
            }
            else {
                (0, utils_1.goToLogin)();
            }
        }
        console.error(new Error(`${res.Body.Fault.Detail?.Error?.Code}: ${res.Body.Fault.Reason?.Text}`));
    }
    if (res.Header?.context) {
        const responseUsedQuota = res.Header.context?.refresh?.mbx?.[0]?.s ??
            res.Header.context?.notify?.[0]?.modified?.mbx?.[0]?.s;
        const _context = normalizeContext(res.Header.context);
        const seq = (0, lodash_1.maxBy)(_context.notify, 'seq')?.seq ?? 0;
        account_1.useAccountStore.setState({
            usedQuota: responseUsedQuota ?? usedQuota
        });
        const nextPollingInterval = (0, utils_2.getPollingInterval)(res);
        network_1.useNetworkStore.setState({
            noOpTimeout: setTimeout(() => fetchNoOp(), nextPollingInterval),
            pollingInterval: nextPollingInterval,
            seq,
            ..._context
        });
    }
};
const handleResponse = (api, res) => {
    handleResponseV2(res);
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    // @ts-ignore
    return res?.Body?.Fault ? res.Body : res.Body[`${api}Response`];
};
/**
 * @deprecated Use soapFetchV2 instead
 */
const getSoapFetch = (app) => (api, body, otherAccount, signal) => (0, fetch_utils_1.soapFetch)(api, body, otherAccount, signal)
    // TODO proper error handling
    .then((res) => handleResponse(api, res))
    .catch((e) => {
    (0, functions_1.report)(app)(e);
    throw e;
});
exports.getSoapFetch = getSoapFetch;
const soapFetchV2 = async (api, body, otherAccount, signal) => {
    const rawSoapResponse = await (0, fetch_utils_1.soapFetch)(api, body, otherAccount, signal);
    // apply side effects
    handleResponseV2(rawSoapResponse);
    return rawSoapResponse;
};
exports.soapFetchV2 = soapFetchV2;
/**
 * @deprecated Use soapFetchV2 instead
 */
const getXmlSoapFetch = (app) => (api, body, otherAccount) => {
    const { zimbraVersion, account } = account_1.useAccountStore.getState();
    return fetch(`/service/soap/${api}Request`, {
        method: 'POST',
        headers: {
            'content-type': 'application/soap+xml'
        },
        body: `<?xml version="1.0" encoding="utf-8"?>
		<soap:Envelope xmlns:soap="http://www.w3.org/2003/05/soap-envelope">
			<soap:Header><context xmlns="urn:zimbra"><userAgent name="${user_agent_1.userAgent}" version="${zimbraVersion}"/>${getXmlSession()}${getXmlAccount(account, otherAccount)}<format type="js"/></context></soap:Header>
			<soap:Body>${body}</soap:Body>
		</soap:Envelope>`
    }) // TODO proper error handling
        .then((res) => res?.json())
        .then((res) => handleResponse(api, res))
        .catch((e) => {
        (0, functions_1.report)(app)(e);
        throw e;
    });
};
exports.getXmlSoapFetch = getXmlSoapFetch;
//# sourceMappingURL=fetch.js.map