"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.exportForTest = void 0;
exports.useLocalStorage = useLocalStorage;
const react_1 = require("react");
const zustand_1 = require("zustand");
const utils_1 = require("../../utils/utils");
function isSameLocalStorageValue(valueA, valueB) {
    return JSON.stringify(valueA) === JSON.stringify(valueB);
}
const useLocalStorageStore = (0, zustand_1.create)()((setState) => ({
    storage: {},
    readValue(key, fallback) {
        const localStorageItem = window.localStorage.getItem(key);
        let item;
        try {
            item = localStorageItem !== null ? JSON.parse(localStorageItem) : fallback;
        }
        catch (error) {
            console.error(`Cannot read local storage ${key} with value "${localStorageItem}":`, error);
            item = fallback;
        }
        setState((state) => {
            if (state.storage[key] === undefined) {
                return { storage: { ...state.storage, [key]: item } };
            }
            return state;
        });
    },
    setValue(key, value) {
        setState((state) => {
            const valueToStore = value instanceof Function ? value(state.storage[key]) : value;
            if (!isSameLocalStorageValue(valueToStore, state.storage[key])) {
                window.localStorage.setItem(key, JSON.stringify(valueToStore));
                return { storage: { ...state.storage, [key]: valueToStore } };
            }
            return state;
        });
    }
}));
const DEFAULT_OPTIONS = {
    keepSyncedWithStorage: true
};
function useLocalStorage(key, initialValue, options = DEFAULT_OPTIONS) {
    const storedValue = useLocalStorageStore((state) => state.storage[key] ?? initialValue);
    const shouldDispatchStorageEventRef = (0, react_1.useRef)(false);
    const localStorageOptions = (0, react_1.useMemo)(() => ({ ...DEFAULT_OPTIONS, ...options }), [options]);
    const readValueForKey = (0, react_1.useCallback)(() => {
        useLocalStorageStore.getState().readValue(key, initialValue);
    }, [key, initialValue]);
    (0, react_1.useEffect)(() => {
        readValueForKey();
    }, [readValueForKey]);
    const setValueForKey = (0, react_1.useCallback)((value) => {
        useLocalStorageStore.getState().setValue(key, value);
        shouldDispatchStorageEventRef.current = true;
    }, [key]);
    (0, react_1.useEffect)(() => {
        if (localStorageOptions?.keepSyncedWithStorage) {
            window.addEventListener('storage', readValueForKey);
        }
        return () => {
            window.removeEventListener('storage', readValueForKey);
        };
    }, [localStorageOptions?.keepSyncedWithStorage, readValueForKey]);
    (0, react_1.useEffect)(() => {
        if (shouldDispatchStorageEventRef.current) {
            // dispatch event when stored value change,
            // but only after render to avoid possible update of parent component during render
            window.dispatchEvent(new Event('storage'));
            shouldDispatchStorageEventRef.current = false;
        }
    }, [storedValue]);
    return [storedValue, setValueForKey];
}
exports.exportForTest = (0, utils_1.createExportForTestOnly)({ useLocalStorageStore });
//# sourceMappingURL=useLocalStorage.js.map